@@ -35,6 +35,7 @@ from gpodder import youtube
 from gpodder import vimeo
 import gpodder
 
+import socket
 import threading
 import urllib
 import urlparse
@@ -341,9 +342,8 @@ class DownloadURLOpener(urllib.FancyURLopener):
         return (None, None)
 
 
-class DownloadQueueWorker(threading.Thread):
+class DownloadQueueWorker(object):
     def __init__(self, queue, exit_callback, continue_check_callback, minimum_tasks):
-        threading.Thread.__init__(self)
         self.queue = queue
         self.exit_callback = exit_callback
         self.continue_check_callback = continue_check_callback
@@ -354,8 +354,11 @@ class DownloadQueueWorker(threading.Thread):
         # download, even if a download limit is in effect.
         self.minimum_tasks = minimum_tasks
 
+    def __repr__(self):
+        return threading.current_thread().getName()
+
     def run(self):
-        logger.info('Starting new thread: %s', self.getName())
+        logger.info('Starting new thread: %s', self)
         while True:
             # Check if this thread is allowed to continue accepting tasks
             # (But only after reducing minimum_tasks to zero - see above)
@@ -366,11 +369,11 @@ class DownloadQueueWorker(threading.Thread):
 
             try:
                 task = self.queue.pop()
-                logger.info('%s is processing: %s', self.getName(), task)
+                logger.info('%s is processing: %s', self, task)
                 task.run()
                 task.recycle()
             except IndexError, e:
-                logger.info('No more tasks for %s to carry out.', self.getName())
+                logger.info('No more tasks for %s to carry out.', self)
                 break
         self.exit_callback(self)
 
@@ -420,10 +423,10 @@ class DownloadQueueManager(object):
                 else:
                     minimum_tasks = 0
 
-                worker = DownloadQueueWorker(self.tasks, self.__exit_callback, \
+                worker = DownloadQueueWorker(self.tasks, self.__exit_callback,
                         self.__continue_check_callback, minimum_tasks)
                 self.worker_threads.append(worker)
-                worker.start()
+                util.run_in_background(worker.run)
 
     def are_queued_or_active_tasks(self):
         with self.worker_threads_access:
@@ -531,6 +534,10 @@ class DownloadTask(object):
             _('Finished'), _('Failed'), _('Cancelled'), _('Paused'))
     (INIT, QUEUED, DOWNLOADING, DONE, FAILED, CANCELLED, PAUSED) = range(7)
 
+    # Wheter this task represents a file download or a device sync operation
+    ACTIVITY_DOWNLOAD, ACTIVITY_SYNCHRONIZE = range(2)
+
+
     def __str__(self):
         return self.__episode.title
 
@@ -553,6 +560,15 @@ class DownloadTask(object):
 
     status_changed = property(fget=__get_status_changed)
 
+    def __get_activity(self):
+        return self.__activity
+
+    def __set_activity(self, activity):
+        self.__activity = activity
+
+    activity = property(fget=__get_activity, fset=__set_activity)
+
+
     def __get_url(self):
         return self.__episode.url
 
@@ -579,6 +595,7 @@ class DownloadTask(object):
     def __init__(self, episode, config):
         assert episode.download_task is None
         self.__status = DownloadTask.INIT
+        self.__activity = DownloadTask.ACTIVITY_DOWNLOAD
         self.__status_changed = True
         self.__episode = episode
         self._config = config
@@ -735,9 +752,40 @@ class DownloadTask(object):
                     self._config.youtube_preferred_fmt_id)
             url = vimeo.get_real_download_url(url)
 
-            downloader =  DownloadURLOpener(self.__episode.channel)
-            headers, real_url = downloader.retrieve_resume(url, \
-                    self.tempname, reporthook=self.status_updated)
+            downloader = DownloadURLOpener(self.__episode.channel)
+
+            # HTTP Status codes for which we retry the download
+            retry_codes = (408, 418, 504, 598, 599)
+            max_retries = max(0, self._config.auto.retries)
+
+            # Retry the download on timeout (bug 1013)
+            for retry in range(max_retries + 1):
+                if retry > 0:
+                    logger.info('Retrying download of %s (%d)', url, retry)
+                    time.sleep(1)
+
+                try:
+                    headers, real_url = downloader.retrieve_resume(url,
+                        self.tempname, reporthook=self.status_updated)
+                    # If we arrive here, the download was successful
+                    break
+                except urllib.ContentTooShortError, ctse:
+                    if retry < max_retries:
+                        logger.info('Content too short: %s - will retry.',
+                                url)
+                        continue
+                    raise
+                except socket.timeout, tmout:
+                    if retry < max_retries:
+                        logger.info('Socket timeout: %s - will retry.', url)
+                        continue
+                    raise
+                except gPodderDownloadHTTPError, http:
+                    if retry < max_retries and http.error_code in retry_codes:
+                        logger.info('HTTP error %d: %s - will retry.',
+                                http.error_code, url)
+                        continue
+                    raise
 
             new_mimetype = headers.get('content-type', self.__episode.mime_type)
             old_mimetype = self.__episode.mime_type
@@ -828,7 +876,7 @@ class DownloadTask(object):
                 self.total_size = util.calculate_size(self.filename)
                 logger.info('Total size updated to %d', self.total_size)
             self.progress = 1.0
-            gpodder.user_hooks.on_episode_downloaded(self.__episode)
+            gpodder.user_extensions.on_episode_downloaded(self.__episode)
             return True
         
         self.speed = 0.0
