@@ -27,12 +27,18 @@ from PySide.QtCore import QAbstractListModel, QModelIndex
 from PySide.QtDeclarative import QDeclarativeView
 
 import os
-import threading
 import signal
 import functools
-import gpodder
 import subprocess
 
+import dbus
+import dbus.service
+
+from dbus.mainloop.glib import DBusGMainLoop
+
+
+import gpodder
+
 _ = gpodder.gettext
 N_ = gpodder.ngettext
 
@@ -63,6 +69,56 @@ EPISODE_LIST_FILTERS = [
     (_('Videos'), 'video'),
 ]
 
+
+class ConfigProxy(QObject):
+    def __init__(self, config):
+        QObject.__init__(self)
+        self._config = config
+
+        config.add_observer(self._on_config_changed)
+
+    def _on_config_changed(self, name, old_value, new_value):
+        if name == 'ui.qml.autorotate':
+            self.autorotateChanged.emit()
+        elif name == 'flattr.token':
+            self.flattrTokenChanged.emit()
+        elif name == 'flattr.flattr_on_play':
+            self.flattrOnPlayChanged.emit()
+
+    def get_autorotate(self):
+        return self._config.ui.qml.autorotate
+
+    def set_autorotate(self, autorotate):
+        self._config.ui.qml.autorotate = autorotate
+
+    autorotateChanged = Signal()
+
+    autorotate = Property(bool, get_autorotate, set_autorotate,
+            notify=autorotateChanged)
+
+    def get_flattr_token(self):
+        return self._config.flattr.token
+
+    def set_flattr_token(self, flattr_token):
+        self._config.flattr.token = flattr_token
+
+    flattrTokenChanged = Signal()
+
+    flattrToken = Property(unicode, get_flattr_token, set_flattr_token,
+            notify=flattrTokenChanged)
+
+    def get_flattr_on_play(self):
+        return self._config.flattr.flattr_on_play
+
+    def set_flattr_on_play(self, flattr_on_play):
+        self._config.flattr.flattr_on_play = flattr_on_play
+
+    flattrOnPlayChanged = Signal()
+
+    flattrOnPlay = Property(bool, get_flattr_on_play, set_flattr_on_play,
+            notify=flattrOnPlayChanged)
+
+
 class Controller(QObject):
     def __init__(self, root):
         QObject.__init__(self)
@@ -71,17 +127,19 @@ class Controller(QObject):
         self.episode_list_title = u''
         self.current_input_dialog = None
         self.root.config.add_observer(self.on_config_changed)
+        self._flattr = self.root.core.flattr
+        self.flattr_button_text = u''
 
     def on_config_changed(self, name, old_value, new_value):
         logger.info('Config changed: %s (%s -> %s)', name,
                 old_value, new_value)
-        if name == 'mygpo_enabled':
+        if name == 'mygpo.enabled':
             self.myGpoEnabledChanged.emit()
-        elif name == 'mygpo_username':
+        elif name == 'mygpo.username':
             self.myGpoUsernameChanged.emit()
-        elif name == 'mygpo_password':
+        elif name == 'mygpo.password':
             self.myGpoPasswordChanged.emit()
-        elif name == 'mygpo_device_caption':
+        elif name == 'mygpo.device.caption':
             self.myGpoDeviceCaptionChanged.emit()
 
     episodeListTitleChanged = Signal()
@@ -97,10 +155,132 @@ class Controller(QObject):
     episodeListTitle = Property(unicode, getEpisodeListTitle, \
             setEpisodeListTitle, notify=episodeListTitleChanged)
 
+    flattrButtonTextChanged = Signal()
+
+    def setFlattrButtonText(self, flattr_button_text):
+        if self.flattr_button_text != flattr_button_text:
+            self.flattr_button_text = flattr_button_text
+            self.flattrButtonTextChanged.emit()
+
+    def getFlattrButtonText(self):
+        return self.flattr_button_text
+
+    flattrButtonText = Property(unicode, getFlattrButtonText,
+            setFlattrButtonText, notify=flattrButtonTextChanged)
+
+    @Slot(QObject)
+    def onPlayback(self, qepisode):
+        if (qepisode.payment_url and self.root.config.flattr.token and
+                self.root.config.flattr.flattr_on_play):
+            success, message = self._flattr.flattr_url(qepisode.payment_url)
+            if not success:
+                logger.warn('Flattr message on playback action: %s', message)
+
+    @Slot(QObject)
+    def updateFlattrButtonText(self, qepisode):
+        self.setFlattrButtonText('')
+
+        if qepisode is None:
+            return
+
+        episode = qepisode._episode
+
+        if not episode.payment_url:
+            return
+        if not self._flattr.has_token():
+            self.setFlattrButtonText(_('Sign in'))
+            return
+
+        @util.run_in_background
+        def get_flattr_info():
+            flattrs, flattred = self._flattr.get_thing_info(episode.payment_url)
+
+            if flattred:
+                self.setFlattrButtonText(_('Flattred (%(count)d)') % {
+                    'count': flattrs
+                })
+            else:
+                self.setFlattrButtonText(_('Flattr this (%(count)d)') % {
+                    'count': flattrs
+                })
+
+    @Slot(QObject)
+    def flattrEpisode(self, qepisode):
+        if not qepisode:
+            return
+
+        episode = qepisode._episode
+
+        if not episode.payment_url:
+            return
+        if not self._flattr.has_token():
+            self.root.show_message(_('Sign in to Flattr in the settings.'))
+            return
+
+        self.root.start_progress(_('Flattring episode...'))
+
+        @util.run_in_background
+        def flattr_episode():
+            try:
+                success, message = self._flattr.flattr_url(episode.payment_url)
+                if success:
+                    self.updateFlattrButtonText(qepisode)
+                else:
+                    self.root.show_message(message)
+            finally:
+                self.root.end_progress()
+
+    @Slot(result=str)
+    def getFlattrLoginURL(self):
+        return self._flattr.get_auth_url()
+
+    @Slot(result=str)
+    def getFlattrCallbackURL(self):
+        return self._flattr.CALLBACK
+
+    @Slot(str)
+    def processFlattrCode(self, url):
+        if not self._flattr.process_retrieved_code(url):
+            self.root.show_message(_('Could not log in to Flattr.'))
+
     @Slot(result='QStringList')
     def getEpisodeListFilterNames(self):
         return [caption for caption, eql in EPISODE_LIST_FILTERS]
 
+    @Slot('QVariant', str)
+    def multiEpisodeAction(self, selected, action):
+        if not selected:
+            return
+
+        count = len(selected)
+        episodes = map(self.root.episode_model.get_object_by_index, selected)
+
+        def delete():
+            for episode in episodes:
+                if not episode.qarchive:
+                    episode.delete_episode()
+            self.update_subset_stats()
+            self.root.mygpo_client.on_delete(episodes)
+            self.root.mygpo_client.flush()
+            for episode in episodes:
+                self.root.on_episode_deleted(episode)
+            self.root.episode_model.sort()
+
+        if action == 'delete':
+            msg = N_('Delete %(count)d episode?', 'Delete %(count)d episodes?', count) % {'count':count}
+            self.confirm_action(msg, _('Delete'), delete)
+        elif action == 'download':
+            for episode in episodes:
+                if episode.qdownloaded:
+                    print '    XXX     already downloaded'
+                    continue
+                episode.qdownload(self.root.config, self.update_subset_stats)
+            self.root.mygpo_client.on_download(episodes)
+            self.root.mygpo_client.flush()
+        elif action == 'play':
+            for episode in episodes:
+                self.root.enqueue_episode(episode)
+
     @Slot(str, result=str)
     def translate(self, x):
         return _(x)
@@ -117,9 +297,18 @@ class Controller(QObject):
     def getVersion(self):
         return gpodder.__version__
 
+    @Slot(result=str)
+    def getReleased(self):
+        return gpodder.__date__
+
+    @Slot(result=unicode)
+    def getCredits(self):
+        credits_file = os.path.join(gpodder.prefix, 'share', 'gpodder', 'credits.txt')
+        return util.convert_bytes(open(credits_file).read())
+
     @Slot(result=unicode)
     def getCopyright(self):
-        return gpodder.__copyright__.decode('utf-8', 'ignore')
+        return util.convert_bytes(gpodder.__copyright__)
 
     @Slot(result=str)
     def getLicense(self):
@@ -150,7 +339,10 @@ class Controller(QObject):
             self.update_subset_stats()
 
         url = episode.get_playback_url()
-        subprocess.Popen(['video-suite', url])
+        if gpodder.ui.harmattan:
+            subprocess.Popen(['video-suite', url])
+        else:
+            util.gui_open(url)
 
         self.root.mygpo_client.on_playback([episode])
         self.root.mygpo_client.flush()
@@ -186,8 +378,7 @@ class Controller(QObject):
             finally:
                 self.root.end_progress()
 
-        t = threading.Thread(target=upload_proc, args=[self])
-        t.start()
+        util.run_in_background(lambda: upload_proc(self))
 
     @Slot()
     def saveMyGpoSettings(self):
@@ -198,10 +389,10 @@ class Controller(QObject):
     myGpoEnabledChanged = Signal()
 
     def getMyGpoEnabled(self):
-        return self.root.config.mygpo_enabled
+        return self.root.config.mygpo.enabled
 
     def setMyGpoEnabled(self, enabled):
-        self.root.config.mygpo_enabled = enabled
+        self.root.config.mygpo.enabled = enabled
 
     myGpoEnabled = Property(bool, getMyGpoEnabled,
             setMyGpoEnabled, notify=myGpoEnabledChanged)
@@ -209,10 +400,10 @@ class Controller(QObject):
     myGpoUsernameChanged = Signal()
 
     def getMyGpoUsername(self):
-        return model.convert(self.root.config.mygpo_username)
+        return model.convert(self.root.config.mygpo.username)
 
     def setMyGpoUsername(self, username):
-        self.root.config.mygpo_username = username
+        self.root.config.mygpo.username = username
 
     myGpoUsername = Property(unicode, getMyGpoUsername,
             setMyGpoUsername, notify=myGpoUsernameChanged)
@@ -220,10 +411,10 @@ class Controller(QObject):
     myGpoPasswordChanged = Signal()
 
     def getMyGpoPassword(self):
-        return model.convert(self.root.config.mygpo_password)
+        return model.convert(self.root.config.mygpo.password)
 
     def setMyGpoPassword(self, password):
-        self.root.config.mygpo_password = password
+        self.root.config.mygpo.password = password
 
     myGpoPassword = Property(unicode, getMyGpoPassword,
             setMyGpoPassword, notify=myGpoPasswordChanged)
@@ -231,10 +422,10 @@ class Controller(QObject):
     myGpoDeviceCaptionChanged = Signal()
 
     def getMyGpoDeviceCaption(self):
-        return model.convert(self.root.config.mygpo_device_caption)
+        return model.convert(self.root.config.mygpo.device.caption)
 
     def setMyGpoDeviceCaption(self, caption):
-        self.root.config.mygpo_device_caption = caption
+        self.root.config.mygpo.device.caption = caption
 
     myGpoDeviceCaption = Property(unicode, getMyGpoDeviceCaption,
             setMyGpoDeviceCaption, notify=myGpoDeviceCaptionChanged)
@@ -298,8 +489,7 @@ class Controller(QObject):
             finally:
                 self.root.end_progress()
 
-        t = threading.Thread(target=merge_proc, args=[self])
-        t.start()
+        util.run_in_background(lambda: merge_proc(self))
 
         for podcast in self.root.podcast_model.get_objects():
             podcast.qupdate(finished_callback=self.update_subset_stats)
@@ -332,6 +522,8 @@ class Controller(QObject):
         elif action.action == 'episode-toggle-archive':
             action.target.toggle_archive()
             self.update_subset_stats()
+        elif action.action == 'episode-delete':
+            self.deleteEpisode(action.target)
         elif action.action == 'episode-enqueue':
             self.root.enqueue_episode(action.target)
         elif action.action == 'mark-as-read':
@@ -432,6 +624,7 @@ class Controller(QObject):
             self.root.mygpo_client.on_delete([episode])
             self.root.mygpo_client.flush()
             self.root.on_episode_deleted(episode)
+            self.root.episode_model.sort()
 
         self.confirm_action(_('Delete this episode?'), _('Delete'), delete)
 
@@ -456,6 +649,10 @@ class Controller(QObject):
 
         toggle_archive = _('Allow deletion') if episode.archive else _('Archive')
         menu.append(helper.Action(toggle_archive, 'episode-toggle-archive', episode))
+
+        if episode.state != gpodder.STATE_DELETED:
+            menu.append(helper.Action(_('Delete'), 'episode-delete', episode))
+
         menu.append(helper.Action(_('Add to play queue'), 'episode-enqueue', episode))
 
         self.show_context_menu(menu)
@@ -484,8 +681,7 @@ class Controller(QObject):
                     self.root.start_progress(_('Adding podcasts...') + ' (%d/%d)' % (idx, len(urls)))
                     try:
                         podcast = self.root.model.load_podcast(url=url, create=True,
-                                max_episodes=self.root.config.max_episodes_per_feed,
-                                mimetype_prefs=self.root.config.mimetype_prefs)
+                                max_episodes=self.root.config.max_episodes_per_feed)
                         podcast.save()
                         self.root.insert_podcast(model.QPodcast(podcast))
                     except Exception, e:
@@ -494,8 +690,7 @@ class Controller(QObject):
             finally:
                 self.root.end_progress()
 
-        t = threading.Thread(target=subscribe_proc, args=[self, urls])
-        t.start()
+        util.run_in_background(lambda: subscribe_proc(self, urls))
 
     @Slot()
     def currentEpisodeChanging(self):
@@ -573,10 +768,32 @@ class gPodderPodcastListModel(gPodderListModel):
         self.reset()
 
 class gPodderEpisodeListModel(gPodderListModel):
-    def __init__(self):
+    def __init__(self, config):
         gPodderListModel.__init__(self)
-        self._filter = 0
+        self._filter = config.ui.qml.state.episode_list_filter
         self._filtered = []
+        self._is_subset_view = False
+
+        self._config = config
+        config.add_observer(self._on_config_changed)
+
+    is_subset_view_changed = Signal()
+
+    def get_is_subset_view(self):
+        return self._is_subset_view
+
+    def set_is_subset_view(self, is_subset_view):
+        if is_subset_view != self.is_subset_view:
+            self._is_subset_view = is_subset_view
+            self.is_subset_view_changed.emit()
+
+    is_subset_view = Property(bool, get_is_subset_view,
+            set_is_subset_view, notify=is_subset_view_changed)
+
+    def _on_config_changed(self, name, old_value, new_value):
+        if name == 'ui.qml.state.episode_list_filter':
+            self._filter = new_value
+            self.sort()
 
     def sort(self):
         caption, eql = EPISODE_LIST_FILTERS[self._filter]
@@ -596,10 +813,17 @@ class gPodderEpisodeListModel(gPodderListModel):
     def get_object(self, index):
         return self._filtered[index.row()]
 
+    @Slot(int, result=QObject)
+    def get_object_by_index(self, index):
+        return self._filtered[int(index)]
+
+    @Slot(result=int)
+    def getFilter(self):
+        return self._filter
+
     @Slot(int)
     def setFilter(self, filter_index):
-        self._filter = filter_index
-        self.sort()
+        self._config.ui.qml.state.episode_list_filter = filter_index
 
 
 def QML(filename):
@@ -623,13 +847,20 @@ class DeclarativeView(QDeclarativeView):
         event.ignore()
 
 class qtPodder(QObject):
-    def __init__(self, args, gpodder_core):
+    def __init__(self, args, gpodder_core, dbus_bus_name):
         QObject.__init__(self)
 
+        self.dbus_bus_name = dbus_bus_name
+        # TODO: Expose the same D-Bus API as the Gtk UI D-Bus object (/gui)
+        # TODO: Create a gpodder.dbusproxy.DBusPodcastsProxy object (/podcasts)
+
         # Enable OpenGL rendering without requiring QtOpenGL
         # On Harmattan we let the system choose the best graphicssystem
-        if '-graphicssystem' not in args and not gpodder.ui.harmattan and not gpodder.win32:
-            args += ['-graphicssystem', 'opengl']
+        if '-graphicssystem' not in args and not gpodder.ui.harmattan:
+            if gpodder.ui.fremantle:
+                args += ['-graphicssystem', 'opengl']
+            elif not gpodder.win32:
+                args += ['-graphicssystem', 'raster']
 
         self.app = QApplication(args)
         signal.signal(signal.SIGINT, signal.SIG_DFL)
@@ -640,12 +871,14 @@ class qtPodder(QObject):
         self.db = self.core.db
         self.model = self.core.model
 
+        self.config_proxy = ConfigProxy(self.config)
+
         # Initialize the gpodder.net client
         self.mygpo_client = my.MygPoClient(self.config)
 
-        gpodder.user_hooks.on_ui_initialized(self.model,
-                self.hooks_podcast_update_cb,
-                self.hooks_episode_download_cb)
+        gpodder.user_extensions.on_ui_initialized(self.model,
+                self.extensions_podcast_update_cb,
+                self.extensions_episode_download_cb)
 
         self.view = DeclarativeView()
         self.view.closing.connect(self.on_quit)
@@ -653,8 +886,9 @@ class qtPodder(QObject):
 
         self.controller = Controller(self)
         self.media_buttons_handler = helper.MediaButtonsHandler()
+        self.tracker_miner_config = helper.TrackerMinerConfig()
         self.podcast_model = gPodderPodcastListModel()
-        self.episode_model = gPodderEpisodeListModel()
+        self.episode_model = gPodderEpisodeListModel(self.config)
         self.last_episode = None
 
         # A dictionary of episodes that are currently active
@@ -667,7 +901,7 @@ class qtPodder(QObject):
         if gpodder.ui.fremantle:
             for path in ('/opt/qtm11/imports', '/opt/qtm12/imports'):
                 engine.addImportPath(path)
-	elif gpodder.win32:
+        elif gpodder.win32:
             for path in (r'C:\QtSDK\Desktop\Qt\4.7.4\msvc2008\imports',):
                 engine.addImportPath(path)
 
@@ -677,8 +911,11 @@ class qtPodder(QObject):
 
         root_context = self.view.rootContext()
         root_context.setContextProperty('controller', self.controller)
+        root_context.setContextProperty('configProxy', self.config_proxy)
         root_context.setContextProperty('mediaButtonsHandler',
                 self.media_buttons_handler)
+        root_context.setContextProperty('trackerMinerConfig',
+                self.tracker_miner_config)
 
         # Load the QML UI (this could take a while...)
         self.view.setSource(QUrl.fromLocalFile(QML('main_default.qml')))
@@ -697,6 +934,14 @@ class qtPodder(QObject):
             self.view.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
             self.view.showFullScreen()
         else:
+            # On the Desktop, scale to fit my small laptop screen..
+            desktop = self.app.desktop()
+            if desktop.height() < 1000:
+                FACTOR = .8
+                self.view.scale(FACTOR, FACTOR)
+                size = self.view.size()
+                size *= FACTOR
+                self.view.resize(size)
             self.view.show()
 
         self.do_start_progress.connect(self.on_start_progress)
@@ -826,10 +1071,12 @@ class qtPodder(QObject):
             # Normal QPodcast instance
             wrap = functools.partial(self.wrap_episode, podcast)
             objects = podcast.get_all_episodes()
+            self.episode_model.set_is_subset_view(False)
         else:
             # EpisodeSubsetView
             wrap = lambda args: self.wrap_episode(*args)
             objects = podcast.get_all_episodes_with_podcast()
+            self.episode_model.set_is_subset_view(True)
 
         self.episode_model.set_objects(map(wrap, objects))
         self.main.state = 'episodes'
@@ -847,26 +1094,36 @@ class qtPodder(QObject):
             return podcasts[0]
         return None
 
-    def hooks_podcast_update_cb(self, podcast):
-        logger.debug('hooks_podcast_update_cb(%s)', podcast)
+    def extensions_podcast_update_cb(self, podcast):
+        logger.debug('extensions_podcast_update_cb(%s)', podcast)
         try:
             qpodcast = self.podcast_to_qpodcast(podcast)
             if qpodcast is not None:
                 qpodcast.qupdate(
                     finished_callback=self.controller.update_subset_stats)
         except Exception, e:
-            logger.exception('hooks_podcast_update_cb(%s): %s', podcast, e)
+            logger.exception('extensions_podcast_update_cb(%s): %s', podcast, e)
 
-    def hooks_episode_download_cb(self, episode):
-        logger.debug('hooks_episode_download_cb(%s)', episode)
+    def extensions_episode_download_cb(self, episode):
+        logger.debug('extensions_episode_download_cb(%s)', episode)
         try:
             qpodcast = self.podcast_to_qpodcast(episode.channel)
             qepisode = self.wrap_episode(qpodcast, episode)
             self.controller.downloadEpisode(qepisode)
         except Exception, e:
-            logger.exception('hooks_episode_download_cb(%s): %s', episode, e)
+            logger.exception('extensions_episode_download_cb(%s): %s', episode, e)
 
 def main(args):
-    gui = qtPodder(args, core.Core())
+    try:
+        dbus_main_loop = DBusGMainLoop(set_as_default=True)
+        gpodder.dbus_session_bus = dbus.SessionBus(dbus_main_loop)
+
+        bus_name = dbus.service.BusName(gpodder.dbus_bus_name,
+                bus=gpodder.dbus_session_bus)
+    except dbus.exceptions.DBusException, dbe:
+        logger.warn('Cannot get "on the bus".', exc_info=True)
+        bus_name = None
+
+    gui = qtPodder(args, core.Core(), bus_name)
     return gui.run()
 
