@@ -31,12 +31,9 @@
 #include <shellapi.h>
 #include <string.h>
 
-#define PROGNAME "gPodder"
-
-#define BAILOUT(s) { \
-    MessageBox(NULL, s, "Error launching " PROGNAME, MB_OK); \
-    exit(1); \
-}
+#include "gpodder.h"
+#include "downloader.h"
+#include "folderselector.h"
 
 #if defined(GPODDER_GUI)
 # define MAIN_MODULE "bin\\gpodder"
@@ -61,14 +58,14 @@
     if(x == NULL) {BAILOUT("Cannot find function: " #x);}}
 
 
-const char *FindPythonDLL()
+const char *FindPythonDLL(HKEY rootKey)
 {
     static char InstallPath[MAX_PATH];
     DWORD InstallPathSize = MAX_PATH;
     HKEY RegKey;
     char *result = NULL;
 
-    if (RegOpenKeyEx(HKEY_CURRENT_USER,
+    if (RegOpenKeyEx(rootKey,
             "Software\\Python\\PythonCore\\2.7\\InstallPath",
             0, KEY_READ, &RegKey) != ERROR_SUCCESS) {
         return NULL;
@@ -90,7 +87,10 @@ int main(int argc, char** argv)
     char path_env[MAX_PATH];
     char current_dir[MAX_PATH];
     char *endmarker = NULL;
-    char *dll_path = NULL;
+    const char *dll_path = NULL;
+    const char *target_folder = NULL;
+    char tmp[MAX_PATH];
+    int force_select = 0;
     int i;
     void *MainPy;
     void *GtkModule;
@@ -112,22 +112,17 @@ int main(int argc, char** argv)
     SetConsoleTitle(PROGNAME);
 #endif
 
-    if (getenv("GPODDER_HOME") == NULL) {
-        /* Get path to the "My Documents" folder */
-        if (SHGetFolderPath(NULL,
-                    CSIDL_PERSONAL | CSIDL_FLAG_CREATE,
-                    NULL,
-                    0,
-                    gPodder_Home) != S_OK) {
-            BAILOUT("Cannot determine your home directory (SHGetFolderPath).");
+    for (i=1; i<argc; i++) {
+        if (strcmp(argv[i], "--select-folder") == 0) {
+            force_select = 1;
         }
+    }
 
-        strncat(gPodder_Home, "\\gPodder\\", MAX_PATH);
-        if (SetEnvironmentVariable("GPODDER_HOME", gPodder_Home) == 0) {
-            BAILOUT("SetEnvironmentVariable for GPODDER_HOME failed.");
-        }
-    } else {
-        strncpy(gPodder_Home, getenv("GPODDER_HOME"), MAX_PATH);
+    DetermineHomeFolder(force_select);
+
+    if (GetEnvironmentVariable("GPODDER_HOME",
+            gPodder_Home, sizeof(gPodder_Home)) == 0) {
+        BAILOUT("Cannot determine download folder (GPODDER_HOME). Exiting.");
     }
     CreateDirectory(gPodder_Home, NULL);
 
@@ -150,7 +145,11 @@ int main(int argc, char** argv)
 
     if (python_dll == NULL) {
         /* Try to detect "just for me"-installed Python version (bug 1480) */
-        dll_path = FindPythonDLL();
+        dll_path = FindPythonDLL(HKEY_CURRENT_USER);
+        if (dll_path == NULL) {
+            /* Try to detect "for all users" Python (bug 1480, comment 9) */
+            dll_path = FindPythonDLL(HKEY_LOCAL_MACHINE);
+        }
         if (dll_path != NULL) {
             python_dll = LoadLibrary(dll_path);
         }
